@@ -10,7 +10,7 @@ use Path::Tiny;
 use Fcntl qw/:seek/;
 use Encode;
 use Regexp::Pattern;
-use Regexp::Pattern::License 3.1.0;
+use Regexp::Pattern::License 3.1.91;
 use String::Copyright 0.003 {
 	format => sub { join ' ', $_->[0] || (), $_->[1] || () }
 };
@@ -31,11 +31,11 @@ App::Licensecheck - functions for a simple license checker for source files
 
 =head1 VERSION
 
-Version v3.0.34
+Version v3.0.35
 
 =cut
 
-our $VERSION = version->declare("v3.0.34");
+our $VERSION = version->declare("v3.0.35");
 
 =head1 SYNOPSIS
 
@@ -381,9 +381,6 @@ sub licensepatterns
 		}
 		$list{name}{$key} ||= $val->{name} || $key;
 		$list{caption}{$key} ||= $val->{caption} || $val->{name} || $key;
-		$list{re}{$key} = re( 'License::' . $key );
-		$list{re}{$key} = qr/$list{re}{$key}/
-			unless ( ref( $list{re}{$key} ) eq 'Regexp' );
 		for ( @{ $val->{tags} } ) {
 			/^(family|type):([a-z][a-z0-9_]*)(?::([a-z][a-z0-9_]*))?/;
 			$list{family}{$2}{$key} = 1
@@ -393,12 +390,17 @@ sub licensepatterns
 			$list{series}{$key} = $3
 				if ( $3 and $1 eq 'type' and $2 eq 'singleversion' );
 		}
+		for my $subject (qw(grant_license name trait)) {
+			my $re = re( "License::$key", subject => $subject =~ tr/_/,/r )
+				or next;
+			$list{"re_$subject"}{$key} = ref($re) ? $re : qr/$re/;
+		}
 	}
 
 	# TODO: use Regexp::Common
-	$list{re}{version}{'-keep'}
-		= qr/(?:$list{re}{version_prefix})?($list{re}{version_number})/i;
-	$list{re}{xgpl}{'-keep'} = qr/(?:the )?(?:GNU )?([AL]?GPL)/i;
+	$list{re_trait_keep}{version}
+		= qr/(?:$list{re_trait}{version_prefix})?($list{re_trait}{version_number})/i;
+	$list{re_name_keep}{gnu} = qr/(?:the )?(?:GNU )?([AL]?GPL)/i;
 
 	return %list;
 }
@@ -467,19 +469,19 @@ sub parse_license
 
 	# version of AGPL/GPL
 	given ($licensetext) {
-		when ( /version ($L{re}{version_number})[.,]? (?:\(?only\)?.? )?(?:of $L{re}{gnu}|(as )?published by the Free Software Foundation)/i ) {
+		when ( /version ($L{re_trait}{version_number})[.,]? (?:\(?only\)?.? )?(?:of $L{re_name}{gnu}|(as )?published by the Free Software Foundation)/i ) {
 			$gplver      = " (v$1)";
 			@spdx_gplver = ($1)
 		}
-		when ( /$L{re}{gnu}\b[;,] $L{re}{version}{-keep}\b[.,]? /i ) {
+		when ( /$L{re_name}{gnu}\b[;,] $L{re_trait_keep}{version}\b[.,]? /i ) {
 			$gplver      = " (v$1)";
 			@spdx_gplver = ($1);
 		}
-		when ( /either $L{re}{version}{-keep}(?: of the License)?,? $L{re}{version_later_postfix}/ ) {
+		when ( /either $L{re_trait_keep}{version}(?: of the License)?,? $L{re_trait}{version_later_postfix}/ ) {
 			$gplver      = " (v$1 or later)";
 			@spdx_gplver = ( $1 . '+' );
 		}
-		when ( /GPL as published by the Free Software Foundation, version ($L{re}{version_number})/i ) {
+		when ( /GPL as published by the Free Software Foundation, version ($L{re_trait}{version_number})/i ) {
 			$gplver      = " (v$1)";
 			@spdx_gplver = ($1)
 		}
@@ -515,11 +517,11 @@ sub parse_license
 	given ($licensetext) {
 		my @multilicenses;
 		# same sentence
-		when ( /$under_terms_of $any_of(?:[^.]|\.\S)* $L{re}{lgpl}(?:[,;:]? ?$L{re}{version}{-keep}(?: of the License)?($L{re}{version_later})?)?/i ) {
+		when ( /$under_terms_of $any_of(?:[^.]|\.\S)* $L{re_name}{lgpl}(?:[,;:]? ?$L{re_trait_keep}{version}(?: of the License)?($L{re_trait}{version_later})?)?/i ) {
 			push @multilicenses, 'lgpl', $1, $2;
 			continue;
 		}
-		when ( /$under_terms_of $any_of(?:[^.]|\.\S)* $L{re}{gpl}(?:[,;:]? ?$L{re}{version}{-keep}(?: of the License)?($L{re}{version_later})?)?/i ) {
+		when ( /$under_terms_of $any_of(?:[^.]|\.\S)* $L{re_name}{gpl}(?:[,;:]? ?$L{re_trait_keep}{version}(?: of the License)?($L{re_trait}{version_later})?)?/i ) {
 			push @multilicenses, 'gpl', $1, $2;
 			continue;
 		}
@@ -529,28 +531,28 @@ sub parse_license
 	# LGPL
 	given ($licensetext) {
 		# LGPL, among several
-		when ( /$under_terms_of $any_of(?:[^.]|\.\S)* $L{re}{lgpl}(?:[,;:]? ?$L{re}{version}{-keep}(?: of the License)?($L{re}{version_later})?)?/i ) {
+		when ( /$under_terms_of $any_of(?:[^.]|\.\S)* $L{re_name}{lgpl}(?:[,;:]? ?$L{re_trait_keep}{version}(?: of the License)?($L{re_trait}{version_later})?)?/i ) {
 			break; # handled in multi-licensing loop
 		}
 		# AFL or LGPL
-		when ( /either $L{re}{afl}(?:[,;]? ?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?, or $L{re}{lgpl}(?:[,;]? ?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/i ) {
+		when ( /either $L{re_name}{afl}(?:[,;]? ?$L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?, or $L{re_name}{lgpl}(?:[,;]? ?$L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?/i ) {
 			break; # handled in AFL loop
 		}
 		# GPL or LGPL
-		when ( /either $L{re}{gpl}(?:[,;]? ?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?(?: \((?:the )?"?GPL"?\))?, or $L{re}{lgpl}/i ) {
+		when ( /either $L{re_name}{gpl}(?:[,;]? ?$L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?(?: \((?:the )?"?GPL"?\))?, or $L{re_name}{lgpl}/i ) {
 			break; # handled in GPL loop
 		}
 		# LGPL, version first
-		when ( /$under_terms_of $L{re}{version}{-keep}( $L{re}{version_later_postfix})? of $L{re}{lgpl}/i ) {
+		when ( /$under_terms_of $L{re_trait_keep}{version}( $L{re_trait}{version_later_postfix})? of $L{re_name}{lgpl}/i ) {
 			$gen_license->( 'lgpl', $1, $2 );
 		}
 		# LGPL, dual versions last
-		when ( /$under_terms_of $L{re}{lgpl}\b[,;:]?(?: either)? ?$L{re}{version}{-keep}(?: of the License)?,? $or_option_re $L{re}{version}{-keep}/i ) {
+		when ( /$under_terms_of $L{re_name}{lgpl}\b[,;:]?(?: either)? ?$L{re_trait_keep}{version}(?: of the License)?,? $or_option_re $L{re_trait_keep}{version}/i ) {
 			$license = "LGPL (v$1 or v$2) $license";
 			push @spdx_license, "LGPL-$1 or LGPL-$2";
 		}
 		# LGPL, version last
-		when ( /$under_terms_of $L{re}{lgpl}(?:[,;:]?(?: either)? ?$L{re}{version}{-keep}(?: of the License)?($L{re}{version_later})?)?/i ) {
+		when ( /$under_terms_of $L{re_name}{lgpl}(?:[,;:]?(?: either)? ?$L{re_trait_keep}{version}(?: of the License)?($L{re_trait}{version_later})?)?/i ) {
 			$gen_license->( 'lgpl', $1, $2 );
 		}
 	}
@@ -558,7 +560,7 @@ sub parse_license
 
 	# AGPL
 	given ($licensetext) {
-		when ( /is free software.? you can redistribute (it|them) and[ \/]or modify (it|them) under the terms of $L{re}{agpl}/i ) {
+		when ( /is free software.? you can redistribute (it|them) and[ \/]or modify (it|them) under the terms of $L{re_name}{agpl}/i ) {
 			$license = "AGPL$gplver$extrainfo $license";
 			push @spdx_license, $gen_spdx->('AGPL');
 		}
@@ -571,10 +573,10 @@ sub parse_license
 			break;
 		}
 		# exclude MPL-2.0 license
-		when ( /means either ([^.]+$L{re}{version_number})+, the GNU Affero General Public License/i ) {
+		when ( /means either ([^.]+$L{re_trait}{version_number})+, the GNU Affero General Public License/i ) {
 			break;
 		}
-		when ( /AFFERO GENERAL PUBLIC LICENSE(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/i ) {
+		when ( /AFFERO GENERAL PUBLIC LICENSE(?:,? $L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?/i ) {
 			$gen_license->( 'agpl', $1, $2 );
 		}
 	}
@@ -603,20 +605,20 @@ sub parse_license
 			break;
 		}
 		# GPL, among several
-		when ( /$under_terms_of $any_of(?:[^.]|\.\S)* $L{re}{gpl}(?:[,;:]? ?$L{re}{version}{-keep}(?: of the License)?($L{re}{version_later})?)?/i ) {
+		when ( /$under_terms_of $any_of(?:[^.]|\.\S)* $L{re_name}{gpl}(?:[,;:]? ?$L{re_trait_keep}{version}(?: of the License)?($L{re_trait}{version_later})?)?/i ) {
 			break; # handled in multi-licensing loop
 		}
 		# GPL or LGPL
-		when ( /either $L{re}{gpl}(?:[,;]? ?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?(?: \((?:the )?"?GPL"?\))?, or $L{re}{lgpl}(?:[,;]? ?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/i ) {
+		when ( /either $L{re_name}{gpl}(?:[,;]? ?$L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?(?: \((?:the )?"?GPL"?\))?, or $L{re_name}{lgpl}(?:[,;]? ?$L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?/i ) {
 			$gen_license->( 'gpl', $1, $2, 'lgpl', $3, $4 );
 		}
 		if ( $gplver or $extrainfo ) {
-			when ( /under (?:the terms of )?(?:version \S+ (?:\(?only\)? )?of )?$L{re}{gpl}/i ) {
+			when ( /under (?:the terms of )?(?:version \S+ (?:\(?only\)? )?of )?$L{re_name}{gpl}/i ) {
 				$license = "GPL$gplver$extrainfo $license";
 				push @spdx_license, $gen_spdx->('GPL');
 			}
 		}
-		when ( /under (?:the terms of )?(?:version \S+ (?:\(?only\)? )?of )?$L{re}{gpl}(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/i ) {
+		when ( /under (?:the terms of )?(?:version \S+ (?:\(?only\)? )?of )?$L{re_name}{gpl}(?:,? $L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?/i ) {
 			$gen_license->( 'gpl', $1, $2 );
 		}
 	}
@@ -625,11 +627,11 @@ sub parse_license
 	# CC
 	given ($licensetext) {
 		foreach my $id ( keys %{ $L{family}{cc} } ) {
-			when ( /$L{re}{$id}(?i: version)? ($L{re}{version_number}) or ($L{re}{version_number})/i ) {
+			when ( /$L{re_name}{$id}(?i: version)? ($L{re_trait}{version_number}) or ($L{re_trait}{version_number})/i ) {
 				$license = "$L{caption}{$id} (v$1 or v$2) $license";
 				push @spdx_license, "$L{name}{$id}-$1 or $L{name}{$id}-$1";
 			}
-			when ( /$L{re}{$id}(?: (?:$L{re}{version}{-keep})?)(?: License)?($L{re}{version_later})?(?:,? (?:and|or) $L{re}{xgpl}{-keep}(?:-?($L{re}{version_number})(,? $L{re}{version_later_postfix})?)?)?/i ) {
+			when ( /$L{re_name}{$id}(?: (?:$L{re_trait_keep}{version})?)(?: License)?($L{re_trait}{version_later})?(?:,? (?:and|or) $L{re_name_keep}{gnu}(?:-?($L{re_trait}{version_number})(,? $L{re_trait}{version_later_postfix})?)?)?/i ) {
 				$gen_license->( $id, $1, $2, $3, $4 );
 			}
 		}
@@ -639,13 +641,13 @@ sub parse_license
 	# BSD
 	if ( $licensetext =~ /THIS SOFTWARE IS PROVIDED .*AS IS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY/ ) {
 		given ($licensetext) {
-			when ( /$L{re}{clause_advertising}/i ) {
+			when ( /$L{re_trait}{clause_advertising}/i ) {
 				$gen_license->('bsd_4_clause');
 			}
-			when ( /$L{re}{clause_non_endorsement}/i ) {
+			when ( /$L{re_trait}{clause_non_endorsement}/i ) {
 				$gen_license->('bsd_3_clause');
 			}
-			when ( /$L{re}{clause_reproduction}/i ) {
+			when ( /$L{re_trait}{clause_reproduction}/i ) {
 				$gen_license->('bsd_2_clause');
 			}
 			default {
@@ -663,13 +665,13 @@ sub parse_license
 
 	# Artistic
 	given ($licensetext) {
-		when ( $L{re}{perl} ) {
+		when ( $L{re_grant_license}{perl} ) {
 			$gen_license->('perl');
 		}
-		when ( $L{re}{artistic_2} ) {
+		when ( $L{re_grant_license}{artistic_2} ) {
 			$gen_license->('artistic_2');
 		}
-		when ( /$L{re}{artistic}(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/ ) {
+		when ( /$L{re_name}{artistic}(?:,? $L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?/ ) {
 			$gen_license->('artistic', $1, $2);
 		}
 	}
@@ -679,26 +681,26 @@ sub parse_license
 	given ($licensetext) {
 
 		# skip referenced license
-		when ($L{re}{rpsl}) {
+		when ($L{re_grant_license}{rpsl}) {
 			break;
 		}
 
-		when ( /$L{re}{apache}(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]+\))*,? or $L{re}{gpl}(?: $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/i ) {
+		when ( /$L{re_name}{apache}(?:,? $L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]+\))*,? or $L{re_name}{gpl}(?: $L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?/i ) {
 			$gen_license->( 'apache', $1, $2, 'gpl', $3, $4 );
 		}
-		when ( /$L{re}{apache}(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]\))*,? or(?: the)? bsd(?:[ -](\d)-clause)?\b/i ) {
+		when ( /$L{re_name}{apache}(?:,? $L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]\))*,? or(?: the)? bsd(?:[ -](\d)-clause)?\b/i ) {
 			$gen_license->( 'apache', $1, $2, "bsd_${3}_clause" );
 		}
-		when ( /$L{re}{apache}(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]\))*,? or $L{re}{mit_new}\b/i ) {
+		when ( /$L{re_name}{apache}(?:,? $L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]\))*,? or $L{re_grant_license}{mit_new}\b/i ) {
 			$gen_license->( 'apache', $1, $2, 'mit_new', $3, $4 );
 		}
-		when ( /$L{re}{apache}(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]\))*,? or $L{re}{mit}\b/i ) {
+		when ( /$L{re_name}{apache}(?:,? $L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?(?:(?: or)? [^ ,]*?apache[^ ,]*| \([^(),]\))*,? or $L{re_name}{mit}\b/i ) {
 			$gen_license->( 'apache', $1, $2, 'mit', $3, $4 );
 		}
-		when ( /$L{re}{apache}(?:,? $L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/i ) {
+		when ( /$L{re_name}{apache}(?:,? $L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?/i ) {
 			$gen_license->( 'apache', $1, $2 );
 		}
-		when ( m<https?www.apache.org/licenses(?:/LICENSE-($L{re}{version_number}))?>i ) {
+		when ( m<https?www.apache.org/licenses(?:/LICENSE-($L{re_trait}{version_number}))?>i ) {
 			$gen_license->( 'apache', $1 );
 		}
 	}
@@ -706,10 +708,10 @@ sub parse_license
 
 	# FSFUL
 	given ($licensetext) {
-		when ( /$L{re}{fsful}/i ) {
+		when ( /$L{re_grant_license}{fsful}/ ) {
 			$gen_license->('fsful');
 		}
-		when ( /This (\w+)(?: (?:file|script))? is free software; $L{re}{fsf_unlimited}/i ) {
+		when ( /This (\w+)(?: (?:file|script))? is free software; $L{re_trait}{fsf_unlimited}/i ) {
 			$license = "FSF Unlimited ($1 derivation) $license";
 			push @spdx_license, "FSFUL~$1";
 		}
@@ -718,10 +720,10 @@ sub parse_license
 
 	# FSFULLR
 	given ($licensetext) {
-		when ( /$L{re}{fsfullr}/i ) {
+		when ( /$L{re_grant_license}{fsfullr}/ ) {
 			$gen_license->('fsfullr');
 		}
-		when ( /This (\w+)(?: (?:file|script))?  is free software; $L{re}{fsf_unlimited_retention}/i ) {
+		when ( /This (\w+)(?: (?:file|script))?  is free software; $L{re_trait}{fsf_unlimited_retention}/i ) {
 			$license = "FSF Unlimited (with Retention, $1 derivation) $license";
 			push @spdx_license, "FSFULLR~$1";
 		}
@@ -730,47 +732,53 @@ sub parse_license
 
 	# JSON
 	given ($licensetext) {
-		when ( /The Software shall be used for Good, not Evil/ ) {
+		when ( $L{re_grant_license}{json} ) {
 			$gen_license->('JSON');
 		}
 	}
 
 	# PHP
 	given ($licensetext) {
-		when ( /This source file is subject to version ($L{re}{version_number}) of the PHP license/ ) {
+		when ( /This source file is subject to version ($L{re_trait}{version_number}) of the PHP license/ ) {
 			$gen_license->( 'PHP', $1 );
 		}
 	}
 
 	# CECILL
 	given ($licensetext) {
-		when ( $L{re}{cecill_1} ) {
-			$gen_license->('cecill_1');
-		}
-		when ( $L{re}{cecill_1_1} ) {
-			$gen_license->('cecill_1_1');
-		}
-		when ( $L{re}{cecill_2} ) {
-			$gen_license->('cecill_2');
-		}
-		when ( $L{re}{cecill_2_1} ) {
-			$gen_license->('cecill_2_1');
-		}
-		when ( $L{re}{cecill_b} ) {
+		when ( $L{re_grant_license}{cecill_b} ) {
 			$gen_license->('cecill_b');
 		}
-		when ( $L{re}{cecill_c} ) {
+		when ( /$L{re_name}{cecill}-[Bb]/ ) {
+			$gen_license->('cecill_b');
+		}
+		when ( $L{re_grant_license}{cecill_c} ) {
 			$gen_license->('cecill_c');
 		}
-		when ( /$L{re}{cecill}(?:(?:-|\s*$L{re}{version_prefix})($L{re}{version_number}))?/ ) {
+		when ( /$L{re_name}{cecill}-[Cc]/ ) {
+			$gen_license->('cecill_c');
+		}
+		when ( $L{re_grant_license}{cecill_1} ) {
+			$gen_license->('cecill_1');
+		}
+		when ( $L{re_grant_license}{cecill_1_1} ) {
+			$gen_license->('cecill_1_1');
+		}
+		when ( $L{re_grant_license}{cecill_2} ) {
+			$gen_license->('cecill_2');
+		}
+		when ( $L{re_grant_license}{cecill_2_1} ) {
+			$gen_license->('cecill_2_1');
+		}
+		when ( /$L{re_name}{cecill}(?:(?:-|\s*$L{re_trait}{version_prefix})($L{re_trait}{version_number}))?/ ) {
 			$gen_license->( 'cecill', $1 );
 		}
 	}
-	push @custom_done, qw(cecill cecill_1 cecill_1_1 cecill_2 cecill_2_1 cecill_b cecill_c);
+	push @custom_done, qw(cecill cecill_1 cecill_1_1 cecill_2 cecill_2_1 cecill_b cecill_b_1 cecill_c cecill_c_1);
 
 	# public-domain
 	given ($licensetext) {
-		when ( /is in $L{re}{public_domain}/i ) {
+		when ( /$L{re_grant_license}{public_domain}/ ) {
 			$gen_license->('public_domain');
 		}
 	}
@@ -778,11 +786,11 @@ sub parse_license
 
 	given ($licensetext) {
 		# AFL or LGPL
-		when ( /either $L{re}{afl}(?:,? ?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?, or $L{re}{lgpl}(?:,? ?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?)?/i ) {
+		when ( /either $L{re_name}{afl}(?:,? ?$L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?, or $L{re_name}{lgpl}(?:,? ?$L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?)?/i ) {
 			$gen_license->( 'afl', $1, $2, 'lgpl', $3, $4 );
 		}
 		# AFL
-		when ( /Licensed under $L{re}{afl}(?: $L{re}{version}{-keep})?/ ) {
+		when ( /Licensed under $L{re_name}{afl}(?: $L{re_trait_keep}{version})?/ ) {
 			$gen_license->( 'afl', $1 );
 		}
 	}
@@ -790,10 +798,14 @@ sub parse_license
 
 	# BSL
 	given ($licensetext) {
+		when ( /$L{re_grant_license}{bsl_1}/ ) {
+			$gen_license->( 'bsl_1' );
+			push @custom_done, qw(bsl_1);
+		}
 		when ( /Permission is hereby granted, free of charge, to any person or organization obtaining a copy of the software and accompanying documentation covered by this license \(the Software\)/ ) {
 			$gen_license->('BSL');
 		}
-		when ( /Boost Software License(?:[ ,-]+ $L{re}{version}{-keep})?/i ) {
+		when ( /Boost Software License(?:[ ,-]+ $L{re_trait_keep}{version})?/i ) {
 			$gen_license->( 'BSL', $1, $2 );
 		}
 	}
@@ -805,7 +817,7 @@ sub parse_license
 		foreach my $id ( sort keys %{ $L{type}{singleversion} } ) {
 			next if ( grep { $id eq $_ } @custom_done );
 
-			when ( $L{re}{$id} ) {
+			when ( $L{re_grant_license}{$id} ) {
 				$gen_license->($id);
 				push @custom_done, $L{series}{$id}
 					if ( $L{series}{$id} );
@@ -819,15 +831,25 @@ sub parse_license
 
 			# skip embedded or referenced licenses
 			if ( grep { $id eq $_ } qw(mpl python) ) {
-				next if $licensetext =~ $L{re}{rpsl};
+				next if $licensetext =~ $L{re_grant_license}{rpsl};
 			}
 
-			when (
-				/$L{re}{$id}(?:\W*\(?$L{re}{version}{-keep}(,? $L{re}{version_later_postfix})?\)?)?/
-				)
-			{
-				$gen_license->( $id, $1, $2 );
-				continue;
+			if ( $L{re_name}{$id} ) {
+				when (
+					/$L{re_name}{$id}(?:\W*\(?$L{re_trait_keep}{version}(,? $L{re_trait}{version_later_postfix})?\)?)?/
+					)
+				{
+					$gen_license->( $id, $1, $2 );
+					push @custom_done, $id;
+					continue;
+				}
+			}
+			next if ( grep { $id eq $_ } @custom_done );
+			if ( $L{re_grant_license}{$id} ) {
+				when (/$L{re_grant_license}{$id}/) {
+					$gen_license->($id);
+					continue;
+				}
 			}
 		}
 
@@ -837,17 +859,18 @@ sub parse_license
 
 			# skip embedded or referenced licenses
 			if ( grep { $id eq $_ } qw(zlib) ) {
-				next if $licensetext =~ $L{re}{cube};
+				next if $licensetext =~ $L{re_grant_license}{cube};
 			}
 			if ( grep { $id eq $_ } qw(ntp) ) {
-				next if $licensetext =~ $L{re}{ntp_disclaimer};
-				next if $licensetext =~ $L{re}{dsdp};
+				next if $licensetext =~ $L{re_grant_license}{ntp_disclaimer};
+				next if $licensetext =~ $L{re_grant_license}{dsdp};
 			}
 			if ( grep { $id eq $_ } qw(ntp_disclaimer) ) {
-				next if $licensetext =~ $L{re}{mit_cmu};
+				next if $licensetext =~ $L{re_grant_license}{mit_cmu};
 			}
 
-			when ( $L{re}{$id} ) {
+#			when ( $L{re}{$id} ) {
+			when ( $L{re_grant_license}{$id} ) {
 				$gen_license->($id);
 				continue;
 			}
